#!/usr/bin/env node
'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/* eslint-disable no-use-before-define */

// Dependencies
var netbank = require('./api');

var chalk = require('chalk');
var debug = require('debug')('node-cba-netbank');
var fs = require('fs');
var inquirer = require('inquirer');
var moment = require('moment-timezone');
var Table = require('easy-table');
var yargs = require('yargs');

moment.tz.setDefault('Australia/Sydney');

var msgQuit = 'quit';
var tagQuit = chalk.red('<Quit>');
var tagAccountName = '<name>';
var tagAccountNumber = '<number>';
var tagFrom = '<from>';
var tagTo = '<to>';
var tagExt = '<ext>';
var outputFilenameTemplate = '[' + tagAccountName + '](' + tagAccountNumber + ') [' + tagFrom + ' to ' + tagTo + '].' + tagExt;
var formatSortable = 'YYYY-MM-DD';
var formatAus = 'DD/MM/YYYY';
var formatQifQuickenAUS = 'DD/MM/YY';
var formatQifQuickenUS = 'MM/DD/YY';

var Render = function () {
  function Render() {
    _classCallCheck(this, Render);
  }

  _createClass(Render, null, [{
    key: 'currency',
    value: function currency(amount) {
      return amount >= 0 ? chalk.green.bold('$' + amount.toFixed(2)) : chalk.red.bold('$' + amount.toFixed(2));
    }
  }, {
    key: 'account',
    value: function account(_account) {
      return chalk.bold(_account.name) + ' \t(' + chalk.red(_account.bsb) + ' ' + chalk.red(_account.account) + ')' + ('\t Balance: ' + Render.currency(_account.balance) + ' ') + ('\t Available Funds: ' + Render.currency(_account.available));
    }
  }, {
    key: 'accounts',
    value: function accounts(_accounts) {
      return Table.print(_accounts.map(function (account) {
        return {
          Name: chalk.bold(account.name),
          Number: chalk.red(account.bsb) + ' ' + chalk.red(account.account),
          Balance: '' + Render.currency(account.balance),
          Available: '' + Render.currency(account.available)
        };
      }));
    }
  }, {
    key: 'transactions',
    value: function transactions(_transactions) {
      return Table.print(_transactions.map(function (t) {
        return {
          Time: chalk.italic.cyan(moment(t.timestamp).format('YYYY-MM-DD HH:mm')),
          Description: t.pending ? chalk.gray(t.description) : chalk.white(t.description),
          Amount: Render.currency(t.amount),
          Balance: t.pending ? '' : Render.currency(t.balance)
        };
      }));
    }
  }, {
    key: 'csvAmount',
    value: function csvAmount(n) {
      return n > 0 ? '+' + n.toFixed(2) : '' + n.toFixed(2);
    }
  }, {
    key: 'csvTransaction',
    value: function csvTransaction(t) {
      return '' + moment(t.timestamp).format(formatAus) + (',"' + Render.csvAmount(t.amount) + '"') + (',"' + t.description + '"') + (',"' + Render.csvAmount(t.balance) + '"\r\n');
    }
  }, {
    key: 'csvTransactions',
    value: function csvTransactions(transactions) {
      return transactions.map(Render.csvTransaction).join('');
    }
  }, {
    key: 'qifMyobTransaction',
    value: function qifMyobTransaction(t) {
      return 'D' + moment(t.timestamp).format(formatAus) + '\r\n' + ('T' + t.amount.toFixed(2) + '\r\n') + ('P' + t.description + '\r\n') + ('L' + (t.amount >= 0 ? 'DEP' : 'DEBIT') + '\r\n') + '^\r\n';
    }
  }, {
    key: 'qifQuicken2004Transaction',
    value: function qifQuicken2004Transaction(t, format) {
      return 'D' + moment(t.timestamp).format(format) + '\r\nT' + t.amount.toFixed(2) + '\r\nP' + t.description + '\r\n^\r\n';
    }
  }, {
    key: 'qifTransactions',
    value: function qifTransactions(transactions, type) {
      var render = void 0;
      switch (type) {
        case 'aus':
          render = function render(t) {
            return Render.qifQuicken2004Transaction(t, formatQifQuickenAUS);
          };
          break;
        case 'us':
          render = function render(t) {
            return Render.qifQuicken2004Transaction(t, formatQifQuickenUS);
          };
          break;
        default:
        case 'myob':
          render = Render.qifMyobTransaction;
          break;
      }
      return '!Type:Bank\r\n' + transactions.map(render).join('');
    }
  }]);

  return Render;
}();

/* eslint-disable class-methods-use-this */


var UI = function () {
  function UI() {
    _classCallCheck(this, UI);

    this.accounts = [];
  }

  _createClass(UI, [{
    key: 'start',
    value: function start(argv) {
      var _this = this;

      return new Promise(function (resolve) {
        //  check given credential
        var questions = [];

        //  client number
        if (!argv.username) {
          questions.push({
            type: 'input',
            name: 'username',
            message: 'Client number:',
            validate: function validate(input) {
              return (/^\d{1,8}$/.test(input)
              );
            }
          });
        }

        //  password
        if (!argv.password) {
          questions.push({
            type: 'password',
            name: 'password',
            message: 'Password:',
            validate: function validate(input) {
              return (/^[\w\d-]{4,16}$/.test(input)
              );
            }
          });
        }

        return resolve(questions.length > 0 ? inquirer.prompt(questions) : argv);
      }).then(function (c) {
        return _this.logon(c);
      }).then(function () {
        return _this.chooseAccountAndShowHistory(argv.months);
      }).catch(function (error) {
        debug(error);
        console.log('Bye!');
      });
    }
  }, {
    key: 'logon',
    value: function logon(credential) {
      var _this2 = this;

      console.log('Logon as account ' + credential.username + ' ...');
      return netbank.logon(credential).catch(function () {
        return inquirer.prompt({
          type: 'confirm',
          name: 'tryagain',
          message: 'Failed to logged in, try again?'
        }).then(function (answer) {
          if (answer.tryagain) {
            console.log();
            return _this2.start(credential);
          }
          throw new Error(msgQuit);
        });
      }).then(function (resp) {
        _this2.accounts = resp.accounts;
        return _this2.accounts;
      });
    }
  }, {
    key: 'chooseAccountAndShowHistory',
    value: function chooseAccountAndShowHistory(months) {
      var _this3 = this;

      return this.selectAccount().then(function (account) {
        return _this3.downloadHistoryAndShow(account, moment().subtract(months, 'months').format(formatAus));
      }).then(function () {
        return _this3.chooseAccountAndShowHistory(months);
      });
    }
  }, {
    key: 'selectAccount',
    value: function selectAccount() {
      var _this4 = this;

      return inquirer.prompt({
        type: 'list',
        name: 'account',
        message: 'Which account?',
        choices: this.accounts.map(function (a) {
          return { name: Render.account(a), value: a };
        }).concat([tagQuit])
      }).then(function (answer) {
        if (answer.account === tagQuit) {
          throw new Error(msgQuit);
        }
        var account = _this4.accounts.find(function (v) {
          return answer.account.number === v.number;
        });
        if (!account) {
          throw new Error(msgQuit);
        }
        debug('selectAccount(' + Render.account(answer.account) + '): found account ' + Render.account(account));
        return account;
      });
    }
  }, {
    key: 'downloadHistory',
    value: function downloadHistory(account, from, to) {
      console.log('Downloading history [' + from + ' => ' + to + '] ...');
      return netbank.getTransactionHistory(account, from, to);
    }
  }, {
    key: 'downloadHistoryAndShow',
    value: function downloadHistoryAndShow(account, from) {
      var to = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : moment().format(formatAus);

      return this.downloadHistory(account, from, to).then(function (history) {
        var allTransactions = history.pendings.map(function (t) {
          return Object.assign({}, t, { pending: true });
        }).concat(history.transactions);
        console.log(Render.transactions(allTransactions));
        console.log('Total ' + history.transactions.length + ' transactions and ' + history.pendings.length + ' pending transactions.');
        return history;
      });
    }
  }]);

  return UI;
}();

var ui = new UI();
var myArgv = yargs.usage('CBA Netbank CLI\nUsage: $0 <command> [args]').option('u', {
  alias: 'username',
  demandOption: !process.env.NETBANK_USERNAME,
  default: process.env.NETBANK_USERNAME,
  defaultDescription: '$NETBANK_USERNAME',
  describe: 'client number',
  type: 'string'
}).option('p', {
  alias: 'password',
  demandOption: !process.env.NETBANK_PASSWORD,
  default: process.env.NETBANK_PASSWORD,
  defaultDescription: '$NETBANK_PASSWORD',
  describe: 'password',
  type: 'string'
}).command('list', 'List accounts', function () {}, function (argv) {
  debug('Listing accounts ' + JSON.stringify(argv) + '...');
  ui.logon(argv).then(function (accounts) {
    console.log(Render.accounts(accounts));
  });
}).command('download', 'Download transactions history for given account', {
  a: {
    alias: 'account',
    demandOption: true,
    describe: 'account name or number',
    type: 'string'
  },
  f: {
    alias: 'from',
    default: moment().subtract(3, 'months').format(formatAus),
    describe: 'history range from date',
    type: 'string'
  },
  t: {
    alias: 'to',
    default: moment().format(formatAus),
    describe: 'history range to date',
    type: 'string'
  },
  o: {
    alias: 'output',
    default: outputFilenameTemplate,
    describe: 'output file name',
    type: 'string'
  },
  format: {
    default: 'json',
    describe: 'the output file format',
    type: 'string',
    choices: ['json', 'csv', 'qif', 'aus.qif', 'us.qif']
  }
}, function (argv) {
  debug('Download transactions ' + JSON.stringify(argv) + '...');
  ui.logon(argv).then(function (accounts) {
    //  matching accounts
    var account = accounts.find(function (a) {
      return a.name.toLowerCase().indexOf(argv.account.toLowerCase()) >= 0 || a.number.indexOf(argv.account) >= 0;
    });
    if (account) {
      debug('' + Render.account(account));
      ui.downloadHistory(account, argv.from, argv.to).then(function (history) {
        console.log('Retrieved ' + history.transactions.length + ' transactions');
        var filename = argv.output.replace(tagAccountName, account.name).replace(tagAccountNumber, account.number).replace(tagFrom, moment(argv.from, formatAus).format(formatSortable)).replace(tagTo, moment(argv.to, formatAus).format(formatSortable)).replace(tagExt, argv.format);
        console.log('filename: ' + filename);
        var content = void 0;
        switch (argv.format) {
          default:
          case 'json':
            content = JSON.stringify(history.transactions);
            break;
          case 'csv':
            content = Render.csvTransactions(history.transactions);
            break;
          case 'qif':
            content = Render.qifTransactions(history.transactions, 'myob');
            break;
          case 'aus.qif':
            content = Render.qifTransactions(history.transactions, 'aus');
            break;
          case 'us.qif':
            content = Render.qifTransactions(history.transactions, 'us');
            break;
        }
        fs.writeFile(filename, content, function (error) {
          if (error) {
            throw error;
          }
        });
      });
    } else {
      console.log('Cannot find account matching pattern \'' + argv.account + '\'');
    }
  });
}).command('ui', 'Interactive user interface.', {
  m: {
    alias: 'months',
    default: 2,
    describe: 'how many months of history should be shown',
    type: 'number'
  }
}, function (argv) {
  debug('UI: ' + JSON.stringify(argv) + '...');
  ui.start(argv).catch(debug);
}).demandCommand(1, 'You have to tell me what to do, right?').help().argv;

debug('argv => ' + JSON.stringify(myArgv));